require "spec_helper"

describe Bindep::Context do
  let (:context) { Bindep::Context.new }
  let (:item) { context.define(:testitem) { } }

  it "fails on init if which tests fails" do
    expect(Bindep::Helpers).to receive(:which_test).once.and_return(false)

    expect_any_instance_of(Bindep::Context).to receive(:error)
    Bindep::Context.new
  end

  it "fails on getting invalid item" do
    expect do
      context.get_item(:invalid)
    end.to raise_error Bindep::Error, "[Bindep] Cannot find item 'invalid' in context!"
  end

  it "can define and get a new item" do
    item = context.define(:testid) do |i|
      i.command = "cmd"
    end   

    expect(item).to be_a Bindep::Item
    expect(item.id).to eq :testid
    expect(item.command).to eq "cmd"

    expect(context.get_item(:testid)).to eq item
  end

  context "checking" do
    it "fails for an non-existing item and no block" do
      expect do
        context.check :testid
      end.to raise_error Bindep::Error
    end

    it "work for an non-existing item and a defining block" do
      expect(context).to receive(:install).once

      context.check(:testid) do |i|
        i.command = "testcmd"
      end

      expect(context.get_item(:testid).command).to eq "testcmd"
    end

    it "works for an existing installed item" do
      context.define(:testid) { |i| i.command = "grep" }

      context.check :testid
    end

    [ false, true ].each do |no_install|
      context "with no_install #{no_install}" do
        before :each do
          context.no_install = no_install
        end

        it "works for an existing installed item with force install" do
          item = context.define(:testid) { |i| i.command = "grep" }

          expect(context).to receive(:install).with(item).once
          context.check :testid, true
        end

        it "works for an existing but not installed item" do
          item = context.define(:testid) { |i| i.command = "foo_bar_not_existing" }

          expect(context).to receive(:install).with(item).once
          context.check :testid
        end
      end
    end

    it "works with dependencies" do
      dep1 = context.define(:dep1) {}
      dep2 = context.define(:dep2) {}
      dep3 = context.define(:dep3) { |i| i.depends = "dep2" }
      item = context.define(:item) { |i| i.depends = [ "dep3", "dep1" ] }

      [ dep2, dep3, dep1, item ].each do |i|
        expect(context).to receive(:install).with(i).once
      end

      context.check :item
    end
  end

  context "pre_install" do
    it "aborts if no_install is set" do
      context.no_install = true

      expect(context).to receive(:error).with(/deactivated/).and_raise(Bindep::Error)

      expect do
        context.send :pre_install, item
      end.to raise_error Bindep::Error
    end

    it "aborts if no install command is found" do
      str = /Cannot find installation instructions/
      expect(context).to receive(:error).with(str).and_raise(Bindep::Error)

      expect do
        context.send :pre_install, item
      end.to raise_error Bindep::Error
    end

    it "outputs command and does nothing else if no_confirm_before_install is set" do
      context.no_confirm_before_install = true

      item.install_command = "my_install -b"

      expect(context).to receive(:puts).with("[Bindep] $ my_install -b\n")

      context.send :pre_install, item
    end

    it "asks for confirmation" do
      item.install_command = "my_install -b"

      expect(context).to receive(:puts).with(/'testitem' is not installed/)
      expect(context).to receive(:puts).with("[Bindep] $ my_install -b\n")
      expect(STDIN).to receive(:gets).once.and_return("y\n")

      context.send :pre_install, item
    end

    it "asks for confirmation and aborts if user wants" do
      item.install_command = "my_install -b"

      expect(context).to receive(:puts).twice
      expect(STDIN).to receive(:gets).once.and_return("bla\n")
      expect(context).to receive(:error).with(/Aborting/)

      context.send :pre_install, item
    end
  end

  context "install" do
    it "runs installation command (success)" do
      item.install_command = "echo 1"
      item.command = "cat"

      expect(context).to receive(:pre_install).with(item)
      expect(Bindep::Helpers).to receive(:system).with("echo 1")

      context.send :install, item
    end

    it "runs installation command (success, no_install true)" do
      context.no_install = true

      expect(context).to receive(:pre_install).with(item).and_raise(RuntimeError)

      expect do
        context.send :install, item
      end.to raise_error RuntimeError
    end

    it "runs installation command (fail)" do
      item.install_command = "echo 1"

      expect(context).to receive(:pre_install).with(item)
      expect(Bindep::Helpers).to receive(:system).with("echo 1")
      expect(context).to receive(:error).with(/Installation for 'testitem' failed/)

      context.send :install, item
    end
  end

  it "runs a command" do
    item.command = "mycmd"

    expect(item).to receive(:local_command).at_least(:once).and_return("mycmd")
    expect(Bindep::Helpers).to receive(:cmd).with("mycmd --myparam mydoc", 'stdin', false)
    expect(Bindep::Helpers).to receive(:cmd).with("mycmd", nil, true)
    expect(Bindep::Helpers).to receive(:cmd).with("mycmd --myparam mydoc", nil, false)

    context.run :testitem, [ "--myparam", "mydoc" ], 'stdin', false
    context.run :testitem, "", nil, true
    context.run :testitem, "--myparam mydoc", nil, false
  end

  context "error handler" do
    it 'prints exception and exits' do
      expect(context).to receive(:puts).with("[Bindep] My message").once

      expect do
        context.send :error, "My message"
      end.to raise_error SystemExit
    end

    it 'prints raises exception if silent_exceptions is true' do
      context.silent_exceptions = true

      expect do
        context.send :error, "My message"
      end.to raise_error Bindep::Error, "[Bindep] My message"
    end
  end
end

