require "spec_helper"

describe Bindep::Item do
  let (:item) { Bindep::Item.new :testid }

  it "can create a valid item and has read and writable properties" do
    item.command = "command"
    item.apt = "apt"
    item.brew = "brew"
    item.depends = [ :dep1, :dep2 ]

    expect(item.id).to eq :testid
    expect(item.command).to eq "command"
    expect(item.apt).to eq "apt"
    expect(item.brew).to eq "brew"
    expect(item.depends).to eq [ :dep1, :dep2 ]
  end

  it "can converts it to symbol" do
    item = Bindep::Item.new "testid"

    expect(item.id).to eq :testid
  end

  # tests in this context rely on sensible defaults (e.g. grep, cat are available, foo_bar_not_here is not)
  context "command selection" do
    it "works for empty command" do
      item.command = nil

      expect(item.local_command).to be_nil
    end

    it "does not change @command" do
      item.command = [ "not_here", "cat" ]

      item.local_command

      expect(item.command).to eq [ "not_here", "cat" ]
    end

    it "works with a single correct command" do
      item.command = "grep"

      expect(item.local_command).to eq "grep"
    end

    it "works with a single wrong command" do
      item.command = "foo_bar_not_here"

      expect(item.local_command).to eq nil
    end

    it "works with a two correct commands" do
      item.command = [ "grep", "cat" ]

      expect(item.local_command).to eq "grep"
    end

    it "works with a one correct command at first position" do
      item.command = [ "cat", "foo_bar_not_here" ]

      expect(item.local_command).to eq "cat"
    end

    it "works with a one correct command at second position" do
      item.command = [ "foo_bar_not_here", "cat", "foo_bar_not_here_two" ]

      expect(item.local_command).to eq "cat"
    end

    it "does not run twice" do
      item.command = [ "foo_bar_not_here", "cat" ]

      expect(item.local_command).to eq "cat"

      item.command = "foo_bar_not_here"

      expect(item.local_command).to eq "cat"
      expect(item.command).to eq "foo_bar_not_here"
    end

    it "runs again if reset parameter is given" do
      item.command = [ "foo_bar_not_here", "cat" ]

      expect(item.local_command).to eq "cat"

      item.command = "grep"

      expect(item.local_command(true)).to eq "grep"
    end
  end

  # cannot cover all tests (would be 2^n test cases, n = package handler count),
  # so we choose a few general ones and edge cases
  context "install commands" do
    it "works for no packages given" do
      expect(item.install_command).to be_nil
    end

    it "works for custom command given" do
      cmd = "my_installer -fmgu my_package"
      item.install_command = cmd

      expect(item.install_command).to eq cmd
    end

    it "works all commands given and all available (except yum)" do
      Bindep::OS.instance_variable_set :@command_exists_cache, {
        apt: true, npm: true, brew: true, yum: false, rubygem: true
      }

      item.apt = "p_apt"
      item.yum = "p_yum"
      item.npm = "p_npm"
      item.brew = "p_brew"
      item.rubygem = "p_gem"
      item.install_command = "my package"

      results = [
        Bindep::OS.apt_install("p_apt"),
        Bindep::OS.brew_install("p_brew"),
        Bindep::OS.npm_install("p_npm"),
        Bindep::OS.rubygem_install("p_gem"),
        "my package",
      ]

      expect(item.install_command).to eq results.join("; ")
    end

    it "does not run any of apt or yum if both are available" do
      Bindep::OS.instance_variable_set :@command_exists_cache, {
        apt: true, npm: true, brew: true, yum: true, rubygem: true
      }

      item.apt = "p_apt"
      item.yum = "p_yum"
      item.brew = "p_brew"

      results = [
        Bindep::OS.brew_install("p_brew"),
      ]

      expect(item.install_command).to eq results.join("; ")
    end

    it "works some commands given and some available" do
      Bindep::OS.instance_variable_set :@command_exists_cache, {
        apt: false, npm: true, brew: false, yum: true, rubygem: false
      }

      item.npm = nil
      item.yum = "p_yum"
      item.brew = "p_brew"
      item.rubygem = "p_gem"
      item.install_command = nil

      results = [
        Bindep::OS.yum_install("p_yum"),
      ]

      expect(item.install_command).to eq results.join("; ")
    end
  end
end
